@@ -264,21 +264,6 @@ class DpmSummariser(object):
         @arg occurence_tag: A string to replace the word "occurence" with in the output.
         """
         
-        # decorate plotting functions to turn pylab interactive mode off
-        self.pcolor = pylab_ioff(self.pcolor)
-        self.imshow = pylab_ioff(self.imshow)
-        self.plot_word_document_scatter = pylab_ioff(self.plot_word_document_scatter)
-        self.plot_num_documents_by_top_topics = pylab_ioff(self.plot_num_documents_by_top_topics)
-        self.make_topic_KL_heat_map = pylab_ioff(self.make_topic_KL_heat_map)
-        self.topic_sizes = pylab_ioff(self.topic_sizes)
-        self.topic_sizes_log_scale = pylab_ioff(self.topic_sizes_log_scale)
-        self._histogram = pylab_ioff(self._histogram)
-        self._plot_dist = pylab_ioff(self._plot_dist)
-        self.make_topic_word_intersection_heat_map = pylab_ioff(self.make_topic_word_intersection_heat_map)
-        self.make_topic_document_intersection_heat_map = pylab_ioff(self.make_topic_document_intersection_heat_map)
-        self.make_phi_heat_map = pylab_ioff(self.make_phi_heat_map)
-        self.make_theta_heat_map = pylab_ioff(self.make_theta_heat_map)
-        
         if None == document_ids:
             document_ids = [str(i) for i in xrange(dpm.D)]
         if None == word_ids:
@@ -363,6 +348,7 @@ class DpmSummariser(object):
         savefig('%s-%s.eps' % (self.prefix, tag), format='EPS', **kwargs)
         #savefig('%s-%s.png' % (self.prefix, tag), format='PNG', **kwargs)
 
+    @pylab_ioff
     def pcolor(self, Z, tag, size=1, **kwargs):
         import pylab as P
         P.figure()
@@ -373,6 +359,7 @@ class DpmSummariser(object):
         self.save_fig(tag)
         P.close()
 
+    @pylab_ioff
     def imshow(self, Z, tag, size=1, **kwargs):
         import pylab as P
         dpi = 100.
@@ -386,6 +373,7 @@ class DpmSummariser(object):
         self.save_fig(tag, facecolor='black', edgecolor='black', dpi=dpi)
         P.close()
 
+    @pylab_ioff
     def plot_word_document_scatter(self):
         "Make a scatter plot of number of words against number of documents for each topic."
         logging.info('Plotting word document scatter')
@@ -395,7 +383,9 @@ class DpmSummariser(object):
         P.scatter(
             self.statistics.num_words_per_topic(),
             self.statistics.num_documents_per_topic(),
-            s=sizes
+            s=sizes,
+            color=P.rcParams['patch.facecolor'],
+            edgecolor='k'
         )
         P.xlim(xmin=0)
         P.ylim(ymin=0)
@@ -405,6 +395,7 @@ class DpmSummariser(object):
         self.save_fig('%s-%s-scatter' % (self.word_tag, self.document_tag))
         P.close()
 
+    @pylab_ioff
     def plot_num_documents_by_top_topics(self):
         "Make a plot of number of documents depending on number of topics included."
         logging.info('Plotting number of documents')
@@ -457,6 +448,7 @@ class DpmSummariser(object):
         self.pcolor(Z, '%s-%s-intersections' % (self.topic_tag, self.document_tag))
         return Z
 
+    @pylab_ioff
     def make_phi_heat_map(self):
         "Make a heat map representing the expected phis."
         K = self.statistics.num_topics_used
@@ -466,6 +458,7 @@ class DpmSummariser(object):
         self.save_fig('expected-phi')
         P.close()
 
+    @pylab_ioff
     def make_theta_heat_map(self):
         "Make a heat map representing the expected thetas."
         K = self.statistics.num_topics_used
@@ -487,6 +480,7 @@ class DpmSummariser(object):
         self.make_topic_word_intersection_heat_map()
         self.make_topic_document_intersection_heat_map()
 
+    @pylab_ioff
     def topic_sizes(self):
         'Create a PNG of the topic sizes, i.e. the expected number of occurences from each topic.'
         logging.info('Creating topic size histogram')
@@ -494,11 +488,12 @@ class DpmSummariser(object):
         P.figure()
         P.bar(xrange(self.statistics.num_topics_used), self.dpm.counts.E_n_k[:self.statistics.num_topics_used])
         P.title('%s sizes' % self.topic_tag)
-        P.xlabel('Programs')
-        P.ylabel('Sizes')
+        P.xlabel('programs')
+        P.ylabel('sizes')
         self.save_fig('%s-sizes' % self.topic_tag)
         P.close()
 
+    @pylab_ioff
     def topic_sizes_log_scale(self):
         'Create a PNG of the topic sizes, i.e. the expected number of occurences from each topic.'
         logging.info('Creating topic size histogram (log scale)')
@@ -506,23 +501,34 @@ class DpmSummariser(object):
         P.figure()
         P.bar(xrange(self.statistics.num_topics_used), self.dpm.counts.E_n_k[:self.statistics.num_topics_used], log=True)
         P.title('%s sizes (log scale)' % self.topic_tag)
-        P.xlabel('Programs')
-        P.ylabel('Sizes')
+        P.xlabel('programs')
+        P.ylabel('sizes')
         self.save_fig('%s-sizes-log-scale' % self.topic_tag)
         P.close()
 
-    def _histogram(self, data, count_tag, index_tag):
+    @pylab_ioff
+    def _histogram(self, data, count_tag, index_tag, new_figure=True):
         import pylab as P
-        P.figure()
-        num_bins = max(data)-min(data)
-        while num_bins > 40:
-            num_bins /= 2
-        P.hist(data, bins=num_bins)
-        P.title('\\# %ss in %ss' % (count_tag, index_tag))
+        if new_figure:
+            P.figure()
+        data_min, data_max = min(data), max(data)
+        # make sure not too many bins - keep to maximum of 32
+        log2_size = numpy.log2(data_max - data_min + 1)
+        step = log2_size > 5 and 2**int(log2_size - 4) or 1
+        bin_ranges = numpy.arange(data_min, data_max+2, step) - .5
+        P.hist(data, bins=bin_ranges, align='mid', color=P.rcParams['patch.facecolor'])
+        P.xlim(bin_ranges[0], bin_ranges[-1])
+        xa = P.gca().get_xaxis()                                                
+        ya = P.gca().get_yaxis()                                                 
+        xa.set_major_locator(P.MaxNLocator(integer=True))                          
+        ya.set_major_locator(P.MaxNLocator(integer=True))                                                     
+        P.setp(xa.get_ticklines(), visible=False)
         P.xlabel('\\# %ss' % count_tag)
         P.ylabel('\\# %ss' % index_tag)
-        self.save_fig('%s-per-%s' % (count_tag, index_tag))
-        P.close()
+        if new_figure:
+            P.title('\\# %ss in %ss' % (count_tag, index_tag))
+            self.save_fig('%s-per-%s' % (count_tag, index_tag))
+            P.close()
 
     def histograms(self):
         'Create PNG histograms of the various statistics.'
@@ -547,7 +553,49 @@ class DpmSummariser(object):
           self.topic_tag,
           self.word_tag
         )
+        self.multi_histogram()
 
+
+    @pylab_ioff
+    def multi_histogram(self):
+        """
+        Create a figure of all histograms
+        """
+        import pylab as P
+        fig = P.figure()
+        P.subplot(2,2,1)
+        self._histogram(
+          self.statistics.num_words_per_topic(),
+          self.word_tag,
+          self.topic_tag,
+          new_figure=False
+        )
+        P.subplot(2,2,2)
+        self._histogram(
+          self.statistics.num_topics_per_word(),
+          self.topic_tag,
+          self.word_tag,
+          new_figure=False
+        )
+        P.subplot(2,2,3)
+        self._histogram(
+          self.statistics.num_documents_per_topic(),
+          self.document_tag,
+          self.topic_tag,
+          new_figure=False
+        )
+        P.subplot(2,2,4)
+        self._histogram(
+          self.statistics.num_topics_per_document(),
+          self.topic_tag,
+          self.document_tag,
+          new_figure=False
+        )
+        fig.tight_layout()
+        self.save_fig('associations')
+        P.close()
+
+    @pylab_ioff
     def _plot_dist(self, dist, title, tag):
         import pylab as P
         P.figure()
@@ -580,6 +628,7 @@ class DpmSummariser(object):
         logging.info('Number %ss using topic: %d', self.document_tag, len(topic_documents))
         logging.info('%ss: %s', self.document_tag, ', '.join(self.document_ids[d] for d in topic_documents))
 
+    @pylab_ioff
     def plot_factor_enrichment(self, k):
         'Plots the enrichment this program has for the various factors.'
         import pylab as P
@@ -647,6 +696,7 @@ class DpmInferenceHistory(object):
                 self.history[name].append(fn())
         return -self.history['negative log likelihood'][-1]
 
+    @pylab_ioff
     def plot(self):
         import pylab as P
         for plot, stats in self.stats.iteritems():
